package com.bocloud.mcs.collector.cisco;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.service.internal.NmsCloudVendorService;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.snmp4j.CommunityTarget;
import org.snmp4j.Snmp;
import org.snmp4j.transport.DefaultUdpTransportMapping;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.io.IOException;
import java.util.*;

@Component
public class CiscoIFCollector extends BocloudCollector implements InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(CiscoIFCollector.class);
    Map<String, Object> cache = new HashMap<>();
    @Value("${collector.cisco.switch.interface.enable:true}")
    boolean enable;

    @Autowired
    private CurrentService currentService;

    @Autowired
    NmsCloudVendorService nmsCloudVendorService;

    @Override
    public int getExpire() {
        return 1800000;
    }

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }


    @Scheduled(initialDelay = 1000L, fixedRate = 30000)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect cisco switch interface metrics start...");
        List<MetricFamilySamples> mfs = new ArrayList<>();

        List<String> basicLabels = Arrays.asList("ip", "ifIndex", "ifName", "ifAlias", "ifDescr","vendorId");
        //端口速度
        GaugeMetricFamily ifSpeedMetric = new GaugeMetricFamily("ifSpeed", " ifSpeed An estimate of the interface's current bandwidth in bits per second - 1.3.6.1.2.1.2.2.1.5", basicLabels);
        //当前带宽
        GaugeMetricFamily ifHighSpeedMetric = new GaugeMetricFamily("ifHighSpeed", "1.3.6.1.2.1.31.1.1.1.15", basicLabels);
        //最大传输包字节数
        GaugeMetricFamily ifMtuMetric = new GaugeMetricFamily("ifMtu", "1.3.6.1.2.1.2.2.1.4", basicLabels);

        //输入字节数
        GaugeMetricFamily ifInOctetsMetric = new GaugeMetricFamily("ifInOctets", "ifInOctets The total number of octets received on the interface, including framing characters - 1.3.6.1.2.1.2.2.1.10", basicLabels);
        // 输入非广播包数
        GaugeMetricFamily ifInUcastPktsMetric = new GaugeMetricFamily("ifInUcastPkts", " ifInUcastPkts The number of packets, delivered by this sub-layer to a higher (sub-)layer, which were not addressed to a multicast or broadcast address at this sub-layer - 1.3.6.1.2.1.2.2.1.11", basicLabels);
        //  输入广播包数
        GaugeMetricFamily ifInNUcastPktsMetric = new GaugeMetricFamily("ifInNUcastPkts", "ifInNUcastPkts The number of packets, delivered by this sub-layer to a higher (sub-)layer, which were addressed to a multicast or broadcast address at this sub-layer - 1.3.6.1.2.1.2.2.1.12", basicLabels);
        //  输入包丢弃数
        GaugeMetricFamily ifInDiscardsMetric = new GaugeMetricFamily("ifInDiscards", "ifInDiscards The number of inbound packets which were chosen to be discarded even though no errors had been detected to prevent their being deliverable to a higher-layer protocol - 1.3.6.1.2.1.2.2.1.13", basicLabels);
        //  输入包错误数
        GaugeMetricFamily ifInErrorsMetric = new GaugeMetricFamily("ifInErrors", " ifInErrors For packet-oriented interfaces, the number of inbound packets that contained errors preventing them from being deliverable to a higher-layer protocol - 1.3.6.1.2.1.2.2.1.14", basicLabels);
        //  输入未知协议包数
        GaugeMetricFamily ifInUnknownProtosMetric = new GaugeMetricFamily("ifInUnknownProtos", "ifInUnknownProtos For packet-oriented interfaces, the number of packets received via the interface which were discarded because of an unknown or unsupported protocol - 1.3.6.1.2.1.2.2.1.15", basicLabels);

        //输出字节数
        GaugeMetricFamily ifOutOctetsMetric = new GaugeMetricFamily("ifOutOctets", " ifOutOctets The total number of octets transmitted out of the interface, including framing characters - 1.3.6.1.2.1.2.2.1.16", basicLabels);
        // 输出非广播包数
        GaugeMetricFamily ifOutUcastPktsMetric = new GaugeMetricFamily("ifOutUcastPkts", "ifOutUcastPkts The total number of packets that higher-level protocols requested be transmitted, and which were not addressed to a multicast or broadcast address at this sub-layer, including those that were discarded or not sent - 1.3.6.1.2.1.2.2.1.17", basicLabels);
        //  输出广播包数
        GaugeMetricFamily ifOutNUcastPktsMetric = new GaugeMetricFamily("ifOutNUcastPkts", "ifOutNUcastPkts The total number of packets that higher-level protocols requested be transmitted, and which were addressed to a multicast or broadcast address at this sub-layer, including those that were discarded or not sent - 1.3.6.1.2.1.2.2.1.18", basicLabels);
        //  输出包丢弃数
        GaugeMetricFamily ifOutDiscardsMetric = new GaugeMetricFamily("ifOutDiscards", "ifOutDiscards The number of outbound packets which were chosen to be discarded even though no errors had been detected to prevent their being transmitted - 1.3.6.1.2.1.2.2.1.19", basicLabels);
        //  输出包错误数
        GaugeMetricFamily ifOutErrorsMetric = new GaugeMetricFamily("ifOutErrors", " ifOutErrors For packet-oriented interfaces, the number of outbound packets that could not be transmitted because of errors - 1.3.6.1.2.1.2.2.1.20", basicLabels);
        //  输出队长
        GaugeMetricFamily ifOutQLenMetric = new GaugeMetricFamily("ifOutQLen", "ifOutQLen The length of the output packet queue (in packets). - 1.3.6.1.2.1.2.2.1.21", basicLabels);
        //oid与名称之间的映射关系
        Map<String, String> OIDStrings = new HashMap<>();
        OIDStrings.put("ifIndex", "1.3.6.1.2.1.2.2.1.1");
        OIDStrings.put("ifName", "1.3.6.1.2.1.31.1.1.1.1");
        OIDStrings.put("ifAlias", "1.3.6.1.2.1.31.1.1.1.18");
        OIDStrings.put("ifDescr", "1.3.6.1.2.1.2.2.1.2");
        OIDStrings.put("ifMtu", "1.3.6.1.2.1.2.2.1.4");
        OIDStrings.put("ifSpeed", "1.3.6.1.2.1.2.2.1.5");
        OIDStrings.put("ifHighSpeed", "1.3.6.1.2.1.31.1.1.1.15");

        OIDStrings.put("ifInOctets", "1.3.6.1.2.1.2.2.1.10");
        OIDStrings.put("ifInUcastPkts", "1.3.6.1.2.1.2.2.1.11");
        OIDStrings.put("ifInNUcastPkts", "1.3.6.1.2.1.2.2.1.12");
        OIDStrings.put("ifInDiscards", "1.3.6.1.2.1.2.2.1.13");
        OIDStrings.put("ifInErrors", "1.3.6.1.2.1.2.2.1.14");
        OIDStrings.put("ifInUnknownProtos", "1.3.6.1.2.1.2.2.1.15");

        OIDStrings.put("ifOutOctets", "1.3.6.1.2.1.2.2.1.16");
        OIDStrings.put("ifOutUcastPkts", "1.3.6.1.2.1.2.2.1.17");
        OIDStrings.put("ifOutNUcastPkts", "1.3.6.1.2.1.2.2.1.18");
        OIDStrings.put("ifOutDiscards", "1.3.6.1.2.1.2.2.1.19");
        OIDStrings.put("ifOutErrors", "1.3.6.1.2.1.2.2.1.20");
        OIDStrings.put("ifOutQLen", "1.3.6.1.2.1.2.2.1.21");


        //获取开启告警的思科交换机信息
        BocloudResult result = nmsCloudVendorService.list(CloudProvider.SMIS_FABRIC);
        Assert.isTrue(result.isSuccess(), "nms get cisco vendor error" + result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudVendor> vendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        vendors.forEach(vendor -> {
            String authentication = vendor.getAuthentication();
            JSONObject jsonObject = JSONObject.parseObject(authentication);
            //获取snmp所需的团体名
            String snmpCommunity = (String) jsonObject.get("snmpCommunity");
            snmpCommunity = new DESEncrptor().decrypt(snmpCommunity.trim(), "BocloudCMP");
            //获取snmp的端口号
            Integer snmpPort = (Integer) jsonObject.get("snmpPort");
            //获取交换机ip
            String address = (String) jsonObject.get("address");
            String ip = address.split(":")[1].substring(2);
            Snmp snmp = null;
            try {
                snmp = new Snmp(new DefaultUdpTransportMapping());
                snmp.listen();
            } catch (IOException e) {
                logger.error("snmp create error", e);
            }
            CommunityTarget target = SNMPv2Utils.createDefault(ip, snmpCommunity, snmpPort);
            //      ifIndex     oid     value
            HashMap<String, Map<String, String>> map = SNMPv2Utils.snmpWalk(snmp, target, new ArrayList<>(OIDStrings.values()));
            map.forEach((index, subMap) -> {
                List<String> labelValues = Arrays.asList(ip, index, subMap.get(OIDStrings.get("ifName") + "." + index), subMap.get(OIDStrings.get("ifAlias") + "." + index), subMap.get(OIDStrings.get("ifDescr") + "." + index),vendor.getId().toString());
                ifSpeedMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifSpeed") + "." + index)));
                ifHighSpeedMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifHighSpeed") + "." + index)));
                ifMtuMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifMtu") + "." + index)));

                ifInOctetsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifInOctets") + "." + index)));
                ifInUcastPktsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifInUcastPkts") + "." + index)));
                ifInNUcastPktsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifInNUcastPkts") + "." + index)));
                ifInDiscardsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifInDiscards") + "." + index)));
                ifInErrorsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifInErrors") + "." + index)));
                ifInUnknownProtosMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifInUnknownProtos") + "." + index)));

                ifOutOctetsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifOutOctets") + "." + index)));
                ifOutUcastPktsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifOutUcastPkts") + "." + index)));
                ifOutNUcastPktsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifOutNUcastPkts") + "." + index)));
                ifOutDiscardsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifOutDiscards") + "." + index)));
                ifOutErrorsMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifOutErrors") + "." + index)));
                ifOutQLenMetric.addMetric(labelValues, Double.parseDouble(subMap.get(OIDStrings.get("ifOutQLen") + "." + index)));
            });
            try {
                if (null != snmp) {
                    snmp.close();
                }
            } catch (IOException e) {
                logger.error("snmp close error", e);
            }
        });
        mfs.add(ifSpeedMetric);
        mfs.add(ifHighSpeedMetric);
        mfs.add(ifMtuMetric);
        mfs.add(ifInOctetsMetric);
        mfs.add(ifInUcastPktsMetric);
        mfs.add(ifInNUcastPktsMetric);
        mfs.add(ifInDiscardsMetric);
        mfs.add(ifInErrorsMetric);
        mfs.add(ifInUnknownProtosMetric);
        mfs.add(ifOutOctetsMetric);
        mfs.add(ifOutUcastPktsMetric);
        mfs.add(ifOutNUcastPktsMetric);
        mfs.add(ifOutDiscardsMetric);
        mfs.add(ifOutErrorsMetric);
        mfs.add(ifOutQLenMetric);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect cisco switch interface metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
