package com.bocloud.mcs.collector.smis;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.service.internal.CmpCloudVendorService;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
public class SmisCollector extends BocloudCollector {

    private final Logger logger = LoggerFactory.getLogger(SmisCollector.class);
    private final Map<String, Object> cache = new HashMap<>();


    @Value("${collector.smis.enable:true}")
    private boolean enable;

    @Autowired
    private CmpCloudVendorService cmpCloudVendorService;
    @Autowired
    private ApplicationContext applicationContext;
    @Autowired
    private CurrentService currentService;

    @Scheduled(initialDelay = 6000L, fixedRate = 100000)
    @Async
    @Override
    public void scheduledCollect()  {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }

        logger.info("Collect SMIS metrics start...");
        long start = System.currentTimeMillis();

        List<GaugeMetricFamily> AllMetrics = new ArrayList<>();
        // 获取平台数据
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.SMIS_STORAGE);
        if (bocloudResult.isFailed()) {
            logger.error("Get vendor list failed, error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> vendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        for (CloudVendor vendor : vendors) {
            try {
                String projectName = JSONObject.parseObject(vendor.getAuthentication()).getString("projectName");
                SmisService smisService = applicationContext.getBean(String.format("smisService[%s]", projectName), SmisService.class);
                AllMetrics.addAll(smisService.getStat(vendor));
            } catch (Exception e) {
                logger.error("Collect SMIS[{}] metrics error: ", vendor.getName(), e);
            }
        }
        cache.put(TIME, new Date());
        cache.put(DATA, AllMetrics);
        logger.info("Collect SMIS metrics end! use " + (System.currentTimeMillis() - start) / 1000D + "s");
    }

    @Override
    protected Map<String, Object> getCache() {
        return cache;
    }

}
