package com.bocloud.mcs.collector.tianyicloud;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.tianyicloud.core.TianyiClient;
import com.bocloud.mcs.collector.tianyicloud.model.MetricWithValueModel;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class TianyiCloudCollector extends BocloudCollector implements InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(TianyiCloudCollector.class);
    private String prefix = "tianyi_";
    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.tianyi.enable:true}")
    boolean enable;

    /**
     * 缓存过期时间，30分钟
     *
     * @return
     */
    @Override
    public int getExpire() {
        return 1800000;
    }

    @Scheduled(initialDelay = 1000L, fixedRate = 120000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Date now = new Date();
        logger.info("collect tianyi metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.TIANYI);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        listCloudVendor = listCloudVendor.stream().filter(v -> null != v.getIsAlarm() && v.getIsAlarm()).collect(Collectors.toList());
        for (CloudVendor cloudVendor : listCloudVendor) {
            if (StringUtils.isBlank(cloudVendor.getAuthentication())) {
                continue;
            }
            JSONObject object = JSONObject.parseObject(cloudVendor.getAuthentication());
            String regionId = object.getString("regionId");
            String secretKey = object.getString("secretKey");
            String accessKey = object.getString("accessKey");
            String projectId = object.getString("projectId");
            secretKey = new DESEncrptor().decrypt(secretKey.trim(), "BocloudCMP");
            accessKey = new DESEncrptor().decrypt(accessKey.trim(), "BocloudCMP");

            TianyiClient client = new TianyiClient();
            client.setAccessKey(accessKey);
            client.setRegionId(regionId);
            client.setSecretKey(secretKey);
            client.setProjectId(projectId);

            Map<String,String> models = client.getVms();
            if (models.isEmpty()) {
                 logger.info("collect tianyi metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
                return;
            }
            List<MetricWithValueModel> metricWithValueModels = client.batchGetMetricDataById(models);
            Map<String, List<MetricWithValueModel>> map = metricWithValueModels.stream().filter(m -> {
                return !m.getDatapoints().isEmpty();
            }).collect(Collectors.groupingBy(m -> m.getMetricName()));

            for (Map.Entry<String, List<MetricWithValueModel>> m : map.entrySet()) {
                List<MetricWithValueModel> values = m.getValue();
                for (MetricWithValueModel v : values) {
                    String metricName = m.getKey();
                    if(TianyiClient.agentMetricMap.containsKey(m.getKey())){
                        metricName = TianyiClient.agentMetricMap.get(m.getKey());
                    }
                    GaugeMetricFamily gauge = new GaugeMetricFamily(prefix + v.getNamespace().replace(".", "_").toLowerCase() + "_" + metricName, "help", Arrays.asList("instance_id", "name", "vendor_id"));
                    double val = v.getDatapoints().get(v.getDatapoints().size() - 1).getAverage();
                    gauge.addMetric(Arrays.asList(v.getDimensions().get(0).getValue(), models.get(v.getDimensions().get(0).getValue()), String.valueOf(cloudVendor.getId())), val);
                    mfs.add(gauge);
                }
            }
        }
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect tianyi metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
